@@ -1,32 +1,43 @@
-#!/usr/bin/python3
-"""matrix-py module to add, substract, multiply matrices.
+#! /usr/bin/python3
+# Copyright (C) 2021 Fares Ahmed
+#
+# This file is part of matrix-py.
+#
+# matrix-py is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# matrix-py is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with matrix-py.  If not, see <http://www.gnu.org/licenses/>.
 
-Copyright (C) 2021 Fares Ahmed
+"""Hackable Matrix module written in pure Python + CLI
 
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
+https://github.com/faresahemdb/matrix-py
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+Please refer to the link above for more information
+on how to use the module.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+You can call matrix-py CLI help with:
+$ matrixpy --help
+OR $ python -m matrixpy --help
 """
 
 # pylint: disable=C0103 # Variable name "m" is "iNvAlId-nAmE"
 
-import argparse
-import json
+import argparse as _argparse
+import json as _json
+import random as _random
 
 
-name = 'matrixpy'
-__version__ = '0.5.1'
 __all__     = ['Matrix', 'MatrixError']
+__version__ = '0.6.1'
+__author__  = "Fares Ahmed <faresahmed@zohomail.com>"
 
 
 class MatrixError(Exception):
@@ -34,15 +45,22 @@ class MatrixError(Exception):
 
 
 class Matrix:
-    """Matrix Object: add, sub, mul, and a lot more"""
+    """Matrix Object that support Addition, Substraction, [...]
+    And Capable of manipulation, Hackable
+    """
 
     # Object Creation: START
-    def __init__(self, matrix) -> None:
-        """Initialize matrix object."""
+    def __init__(self, matrix):
+        """Initialize Matrix Object | 3 Ways.
+
+        [Nested List]    Matrix([[1, 2, 3], [4, 5, 6]])
+        [One Number (I)] Matrix(3) -> (3x3) Identity Matrix
+        [String]         Matrix("1 2 3; 4 5 6")
+        """
         # Matrix([[1, 2, 3], [4, 5, 6]]) -> Row1 = (1 2 3), Row2 = (4 5 6)
         self.matrix = matrix
 
-        # Matrix(3) -> (3x3) Zero Matrix
+        # Matrix(3) -> (3x3) Identity Matrix
         if isinstance(matrix, int):
             self.matrix = Matrix.identity(matrix).tolist()
 
@@ -62,7 +80,12 @@ class Matrix:
         self.colsnum = len(self.matrix[0])
 
     def __repr__(self):
-        """Returnt the matrix in string formatted way"""
+        """Returnt a representation of the Matrix Object
+        Appears when using an interactive Python shell
+
+        >>> Matrix(3)
+        Output: '1 0 0; 0 1 0; 0 0 1'
+        """
         result = list()
 
         ma_str = [list(map(str, self.matrix[i]))
@@ -74,16 +97,51 @@ class Matrix:
         return "; ".join(result)
 
     def __str__(self):
-        """Return the Matrix, the size of it (Activated when using print)"""
-        result = list()
+        """Return the matrix in `str` representation
+        Appears when using print(Matrix)
 
-        for row in self.matrix:
-            result.append(" ".join(map(str, row)))
+        >>> print(Matrix.random((3,3), 1, 1000))
+        Output: 133 23  388
+                4   335 6
+                72  8   933
+                   (3x3)
+        """
+        matrix_str = list()
+        rows = str()
 
-        return "\n".join(result) + f"\n({self.rowsnum}x{self.colsnum})"
+        for row in self.matrix:  # [[1, 2, 3]] -> [["1", "2", "3"]]
+            matrix_str.append(list(map(str, row)))
+
+        # Get the maximum number in the matrix
+        maxlen = int()
+        for row in matrix_str:
+            if len(max(row, key=len)) > maxlen:
+                maxlen = len(max(row, key=len))
+
+        for i in range(len(matrix_str)):
+            for x in range(len(matrix_str[0])):
+                rows += matrix_str[i][x] + " "
+                rows += " " * (maxlen-len(matrix_str[i][x]))
+            rows = rows.rstrip()
+            rows += "\n"
+
+        # Calculate the spaces before (ROWSNUMxCOLSNUM)
+        rwcl_spaces = " " * (len(rows.split("\n")[-2]) // 2 -
+                            len(f"({self.rowsnum}x{self.colsnum})") // 2)
+
+        return rows + rwcl_spaces + f"({self.rowsnum}x{self.colsnum})"
 
     def __getitem__(self, rowcol):
-        """Return matrix item | a_matrix[1, 2] 1->row 2->col"""
+        """Return row, col, or item of Matrix Object
+        MatrixObject = Matrix("1 2 3; 4 5 6")
+
+        [Row] MatrixObject[1]  -> '4 5 6'
+        [Col] MatrixObject[:1] -> '2; 5'
+        [Item] MatrixObject[1, 2] -> 6
+
+        Note: in [Item] the first arg is the row
+        num and the second is the col num.
+        """
         # Return row if one argument (int) was given (M[1])
         if isinstance(rowcol, int):
             return Matrix(self.matrix).row(rowcol)
@@ -96,7 +154,7 @@ class Matrix:
         return self.matrix[rowcol[0]][rowcol[1]]
 
     def __contains__(self, item):
-        """Return True if item in Object else False"""
+        """If item `in` MatrixObject | True if in the Matrix else False"""
         for row in self.matrix:
             if item in row:
                 return True
@@ -105,7 +163,7 @@ class Matrix:
 
     # Object Expressions: START
     def __pos__(self):
-        """Positive operator: +A | Return the matrix * 1 (copy)"""
+        """Positive operator: +MatA | Return MatA * 1 (copy)"""
         result = list()
 
         for i in range(self.rowsnum):
@@ -116,7 +174,7 @@ class Matrix:
         return Matrix(result)
 
     def __neg__(self):
-        """Negative operator: -A. | Returns the matrix * -1"""
+        """Negative operator: -MatA. | Returns MatA * -1"""
         result = [[-x for x in y] for y in self.matrix]
 
         return Matrix(result)
@@ -124,9 +182,9 @@ class Matrix:
 
     # Object Math operations: START
     def __add__(self, other):
-        """Matrix Addition: A + B or A + INT."""
+        """Matrix Addition: MatA + MatB or MatA + INT."""
         if isinstance(other, Matrix):
-            # A + B
+            # MatA + MatB
             result = list()
 
             if (self.rowsnum != other.rowsnum or
@@ -140,7 +198,7 @@ class Matrix:
                     result[m].append(self.matrix[m][j] + other.matrix[m][j])
 
         else:
-            # A + INT
+            # MatA + INT
             result = list()
 
             for m in range(self.rowsnum):
@@ -151,9 +209,9 @@ class Matrix:
         return Matrix(result)
 
     def __sub__(self, other):
-        """Matrix Subtraction: A - B or A - INT."""
+        """Matrix Subtraction: MatA - MatB or MatA - INT."""
         if isinstance(other, Matrix):
-            # A + B
+            # MatA + MatB
             result = list()
 
             if (self.rowsnum != other.rowsnum or
@@ -166,7 +224,7 @@ class Matrix:
                 for j in range(self.colsnum):
                     result[m].append(self.matrix[m][j] - other.matrix[m][j])
         else:
-            # A + INT
+            # MatA + INT
             result = list()
 
             for m in range(self.rowsnum):
@@ -177,12 +235,12 @@ class Matrix:
         return Matrix(result)
 
     def __mul__(self, other):
-        """Matrix Multiplication: A * B or A * INT."""
+        """Matrix Multiplication: MatA * MatB or MatA * INT."""
         if isinstance(other, Matrix):
-            # A * B
+            # MatA * MatB
             if self.colsnum != other.rowsnum:
-                raise MatrixError('The number of rows in matrix A must be'
-                ' equal to the number of columns in B matrix') from None
+                raise MatrixError('The number of Columns in MatA must be'
+                ' equal to the number of Rows in MatB') from None
 
             # References:
             # https://www.geeksforgeeks.org/python-program-multiply-two-matrices
@@ -190,7 +248,7 @@ class Matrix:
                             for B_col in zip(*other.matrix)]
                                     for A_row in self.matrix]
         else:
-            # A * INT
+            # MatA * INT
             result = list()
 
             for m in range(self.rowsnum):
@@ -203,7 +261,13 @@ class Matrix:
 
     # Object Manpulation: START
     def row(self, num: int, start=0):
-        """Return the given row position"""
+        """Return the row of the position `num`
+        Alternative to `MatrixObject[INT]` (it's
+        actullay what `MatrixObject[INT]` is using)
+
+        start=1 if you want to use the Matrix like
+        in real life.
+        """
         if num > start-1:
             num -= start
 
@@ -214,7 +278,13 @@ class Matrix:
 
 
     def col(self, num: int, start=0):
-        """Return the given col position"""
+        """Return the col in the position `num`
+        Alternative to `MatrixObject[:INT]` (it's
+        actullay what `MatrixObject[:INT]` is using)
+
+        start=1 if you want to use the Matrix Object
+        like in real life
+        """
         if num > start-1:
             num -= start
 
@@ -225,7 +295,15 @@ class Matrix:
 
 
     def addrow(self, row, index=-1):
-        """Add a row to an exisiting Matrix object"""
+        """Add a new row to your Matrix Object
+        MatrixObject -> '1 2 3'
+
+        MatrixObject.addrow('4 5 6')
+        Output: '1 2 3; 4 5 6'
+
+        DON'T USE IT. IT'S BUGGY AND UNDER
+        DEVOLPMENT RIGHT NOW.
+        """
         result = self.matrix
 
         if index == -1:
@@ -237,7 +315,15 @@ class Matrix:
 
 
     def addcol(self, col, index=-1):
-        """Add a col to an exisiting Matrix object"""
+        """Add a new col to your Matrix Object
+        MatrixObject -> '1 2 3; 5 6 7'
+
+        MatrixObject.addcol('4 8')
+        Output: '1 2 3 4; 5 6 7 8'
+
+        DON'T USE IT. IT'S BUGGY AND UNDER
+        DEVOLPMENT RIGHT NOW.
+        """
         result = self.matrix
 
         if index == -1:
@@ -251,7 +337,15 @@ class Matrix:
 
 
     def rmrow(self, index):
-        """Remove a row from an exisiting Matrix object"""
+        """Remove an Existing row from your Matrix Object.
+        MatrixObject -> '1 2 3; 4 5 6'
+
+        MatrixObject.rmrow(1)
+        Output: '1 2 3'
+
+        DON'T USE IT. IT'S BUGGY AND UNDER
+        DEVOLPMENT RIGHT NOW.
+        """
         result = self.matrix
 
         try:
@@ -263,7 +357,15 @@ class Matrix:
 
 
     def rmcol(self, index):
-        """Remove a col from an exisiting Matrix object"""
+        """Remove an Existing col from your Matrix Object.
+        MatrixObject -> '1 2 3 4; 5 6 7 8'
+
+        MatrixObject.rmcol(1)
+        Output: '1 3 4; 5 7 8'
+
+        DON'T USE IT. IT'S BUGGY AND UNDER
+        DEVOLPMENT RIGHT NOW.
+        """
         result = self.matrix
 
         try:
@@ -276,23 +378,25 @@ class Matrix:
 
 
     def transpose(self: list):
-        """Return a new matrix transposed"""
+        """Return the Matrix transposed
+        rows -> cols, cols -> rows
+        """
         return Matrix([list(i) for i in zip(*self.matrix)])
 
 
     def tolist(self):
-        """Convert Matrix object to a list"""
+        """Convert Matrix Object to a Nested List"""
         return self.matrix
     # Object Manpulation: END
 
     # Booleon Expressions: START
     def is_square(self):
-        """Check if the matrix is square"""
+        """True if the matrix is square else False"""
         return bool(self.rowsnum == self.colsnum)
 
 
     def is_symmetric(self):
-        """Check of the matrix is symmetric"""
+        """True if the matrix is symmetric else False"""
         if self.matrix == (Matrix(self.matrix).transpose()).tolist():
             return True
         return False
@@ -301,7 +405,12 @@ class Matrix:
     # Pre Made Objects: START
     @staticmethod
     def identity(size: int):
-        """Return a New Identity Matrix"""
+        """Return a New Identity Matrix (I)
+        Alternative to `Matrix(INT)`
+
+        I Matrix is always square that's why
+        there's one arg
+        """
         result = list()
 
         for i in range(size):
@@ -313,26 +422,62 @@ class Matrix:
 
     @staticmethod
     def zero(size: int):
-        """Return a New Zero Matrix"""
+        """Return a New Zero Matrix (All it's items = 0)"""
         return Matrix([[0] * size] * size)
 
 
     @staticmethod
     def diagonal(*numbers: int):
-        """"Return a New diagonal Matrix"""
+        """"Return a New diagonal Matrix OR get the
+        diagonal of YOUR MatrixObject.
+        MatrixObject = '1 2; 3 4'
+
+        [New] Matrix.diagonal(1, 2, 3) -> '1 0 0; 0 2 0; 0 0 3'
+        [Get] Matrix.diagonal(MatrixObject) -> '1 4'
+        """
         result = list()
 
+        if isinstance(numbers[0], Matrix):
+            result = numbers[0]
+            return Matrix([[result[i, i] for i in range(result.rowsnum)]])
+
         for i, number in enumerate(numbers):
             result.append([0] * len(numbers))
             result[i][i] = number
 
         return Matrix(result)
+
+
+    @staticmethod
+    def randint(size: tuple, a: int, b: int):
+        """Return random matrix in range [a, b]
+        Using randint from random module to generate
+        a MatrixObject (size) with random numbers
+        in range [a, b]
+
+        print(Matrix.randint((3,3), 1, 100))
+        Output: 1  25 58
+                59 18 48
+                15 6  70
+                  (3x3)
+        """
+        result  = list()
+        rowsnum = size[0]
+        colsnum = size[1]
+
+        for row in range(rowsnum):
+            result.append([])
+            for _ in range(colsnum):
+                result[row].append(_random.randint(a, b))
+
+        return Matrix(result)
     # Pre Made Objects: END
 
 
-def main():
-    """The CLI for the module"""
-    parser=argparse.ArgumentParser(
+def _cli():
+    """The Command-Line Interface (CLI) for the module"""
+
+    parser=_argparse.ArgumentParser(
         prog="matrix-py",
         description = 'matrix-py module to add, substract, multiply'
         'matrices.',
@@ -345,21 +490,21 @@ def main():
     )
 
     parser.add_argument('-s', '--size',
-        type=json.loads,
+        type=_json.loads,
         metavar='',
-        help='Size of A Matrix'
+        help='Size of MatA'
     )
 
     parser.add_argument('-t', '--transpose',
-        type=json.loads,
+        type=_json.loads,
         metavar='',
-        help='Transpose of A Matrix (-t "[[1, 2, 3], [4, 5, 6]]")'
+        help='Transpose of MatA (-t "[[1, 2, 3], [4, 5, 6]]")'
     )
 
     parser.add_argument('-ma', '--matrixa',
-        type=json.loads,
+        type=_json.loads,
         metavar='',
-        help='Matrix A (.. -ma "[[1, 2, 3], [4, 5, 6]]")'
+        help='MatA (.. -ma "[[1, 2, 3], [4, 5, 6]]")'
     )
 
     parser.add_argument('-op', '--operator',
@@ -369,9 +514,9 @@ def main():
     )
 
     parser.add_argument('-mb', '--matrixb',
-        type=json.loads,
+        type=_json.loads,
         metavar='',
-        help='Matrix B (.. -mb "[[1, 2, 3], [4, 5, 6]]")'
+        help='MatB (.. -mb "[[1, 2, 3], [4, 5, 6]]")'
     )
 
     parser.add_argument('-I', '--identity',
@@ -387,7 +532,7 @@ def main():
     )
 
     parser.add_argument('-diag', '--diagonal',
-        type=json.loads,
+        type=_json.loads,
         metavar='',
         help='Diagonal (.. -diag [1, 2, 3, 4])'
     )
@@ -419,8 +564,8 @@ def main():
         else:
             raise SyntaxError('The avillable operations are +, -, *')
     else:
-        return parser.print_help()
+        print(parser.print_help())
 
 
 if __name__ == '__main__':
-    main()
+    _cli()
